import { ChangeDetectorRef, Component, Input } from "@angular/core";
import { FeedbackService } from "./feedback.service";
var CANVAS_ID = "feedback-canvas";
var HIGHLIGHT_CLASS = "feedback-highlight";
var FEEDBACK_BTN_CLASS = "btn-feedback";
var FeedbackScreenshotComponent = (function () {
    function FeedbackScreenshotComponent(changeDetection, feedback) {
        this.changeDetection = changeDetection;
        this.feedback = feedback;
        this.screenshots = [];
        this.isDrawing = false;
        feedback.screenshotComponent = this;
    }
    /**
     * Add transparent background to canvas.
     */
    FeedbackScreenshotComponent.prototype.addCanvasAlphaBackground = function () {
        if (!this.canvasContext) {
            throw Error("User feedback context does not exist");
        }
        this.canvasContext.fillStyle = "rgba(102,102,102,0.5)";
        this.canvasContext.fillRect(0, 0, document.body.clientWidth, document.body.clientHeight);
    };
    /**
     * Add a screenshot from local machine.
     * @param event the event of selecting image.
     */
    FeedbackScreenshotComponent.prototype.addScreenshot = function (event) {
        var _this = this;
        var reader = new FileReader();
        var file = event.target.files[0];
        if (file) {
            reader.onload = function (evnt) {
                _this.screenshots.push(evnt.target.result);
                _this.changeDetection.detectChanges();
                event.target.value = "";
            };
            reader.readAsDataURL(file);
        }
    };
    /**
     * Adjust the drawing according to user movement.
     * @param x the x-axis movement position.
     * @param y the y-axis movement position.
     */
    FeedbackScreenshotComponent.prototype.adjustDrawing = function (x, y) {
        if (this.isDrawing) {
            var width = x - this.centerX;
            var height = y - this.centerY;
            this.canvasContext.clearRect(0, 0, document.body.clientWidth, document.body.clientHeight);
            this.addCanvasAlphaBackground();
            this.canvasContext.clearRect(this.centerX, this.centerY, width, height);
            this.redraw();
        }
    };
    /**
     * Clear all screenshots.
     */
    FeedbackScreenshotComponent.prototype.clearScreenshots = function () {
        this.screenshots = [];
        this.changeDetection.detectChanges();
    };
    /**
     * Create a canvas of the page.
     */
    FeedbackScreenshotComponent.prototype.createCanvas = function () {
        if (!this.canvasContext) {
            var canvas = document.createElement("canvas");
            canvas.setAttribute("id", CANVAS_ID);
            canvas.setAttribute("width", String(document.body.clientWidth));
            canvas.setAttribute("height", String(document.body.clientHeight));
            canvas.setAttribute("style", "top: 0");
            document.body.appendChild(canvas);
            this.canvas = canvas;
            this.canvasContext = canvas.getContext("2d");
            this.setupMouseDrawingEvents();
            this.addCanvasAlphaBackground();
        }
    };
    /**
     * Destroy the page canvas.
     */
    FeedbackScreenshotComponent.prototype.destroyCanvas = function () {
        if (this.canvas) {
            this.canvas.remove();
        }
        var highlights = document.getElementsByClassName(HIGHLIGHT_CLASS);
        /* Remove all page highlights. */
        for (var i = 0; i < highlights.length; i++) {
            var highlight = highlights[i];
            highlight.remove();
        }
        this.canvasContext = null;
    };
    /**
     * Enter the screenshot mode.
     */
    FeedbackScreenshotComponent.prototype.enterScreenshotMode = function () {
        if (this.configuration.onEnterTakeScreenshot) {
            this.configuration.onEnterTakeScreenshot();
        }
        this.feedback.buttonComponent.isHidden = false;
        this.feedback.buttonComponent.isScreenshotMode = true;
        this.feedback.modalComponent.hideModal();
        this.createCanvas();
    };
    /**
     * Finish drawing at given position.
     * @param x the x-axis page position to finish at.
     * @param y the y-axis page position to finish at.
     */
    FeedbackScreenshotComponent.prototype.finishDrawing = function (x, y) {
        this.isDrawing = false;
        var width = x - this.centerX;
        var height = y - this.centerY;
        /* If below 10 pixels, consider just a click.*/
        if (width < 10 && height < 10) {
            return;
        }
        /* Fill drawn canvas (highlight) with transparency. */
        this.canvasContext.fillStyle = "rgba(0,0,0,0)";
        this.canvasContext.strokeRect(this.centerX, this.centerY, width, height);
        this.canvasContext.fillRect(this.centerX, this.centerY, width, height);
        /* Remove old element. */
        if (this.highlight) {
            this.highlight.remove();
        }
        /* Create a highlight document element. */
        this.highlight = document.createElement("div");
        this.highlight.setAttribute("style", "position:absolute;top:" + this.centerY
            + "px;left:" + this.centerX
            + "px;width:" + width
            + "px;height:" + height
            + "px;z-index:30000;");
        this.highlight.setAttribute("class", HIGHLIGHT_CLASS);
        this.highlightLeft = this.centerX;
        this.highlightTop = this.centerY;
        this.highlightWidth = width;
        this.highlightHeight = height;
        /* Add highlight div to body. */
        document.body.appendChild(this.highlight);
        this.redraw();
        if (this.configuration.onHighlightDrawn) {
            this.configuration.onHighlightDrawn(this.highlight);
        }
    };
    /**
     * Process rendered screenshot.
     * @param canvas the canvas.
     */
    FeedbackScreenshotComponent.prototype.processRenderedScreenshot = function (canvas) {
        var _this = this;
        this.feedback.setScreenshotMode(false);
        this.feedback.showFeedbackModal();
        this.feedback.unlockTakeScreenshotButtons();
        this.feedback.hideSendFeedbackButton();
        this.destroyCanvas();
        canvas.style.width = "100%";
        canvas.style.borderRadius = "12px";
        var sizingCanvas = null;
        /* Handle screenshot highlight case. */
        if (this.configuration.screenshotOnlyHighlighted) {
            /* Create canvas of highlight area.*/
            sizingCanvas = document.createElement("canvas");
            sizingCanvas.width = this.highlightWidth;
            sizingCanvas.height = this.highlightHeight;
            sizingCanvas.getContext("2d").drawImage(canvas, -this.highlightLeft, -this.highlightTop, canvas.width, canvas.height);
        }
        setTimeout(function () {
            var screenshotBase64 = _this.configuration.screenshotOnlyHighlighted ? sizingCanvas.toDataURL() : canvas.toDataURL();
            _this.screenshots.push(screenshotBase64);
            _this.changeDetection.detectChanges();
            document.body.scrollTop = _this.preRenderScrollPosition; // Reset scroll.
            document.body.style["overflow"] = null; // Reset scrolling.
            document.body.classList.remove("wait"); // Set body cursor back to normal.
            _this.canvas.classList.remove("wait"); // Set canvas cursor back to normal.
            if (_this.configuration.onScreenshotTaken) {
                _this.configuration.onScreenshotTaken(screenshotBase64);
            }
        });
    };
    /**
     * Redraw the canvas context.
     */
    FeedbackScreenshotComponent.prototype.redraw = function () {
        var highlights = document.getElementsByClassName(HIGHLIGHT_CLASS);
        for (var i = 0; i < highlights.length; i++) {
            var highlight = highlights[i];
            var style = highlight.getAttribute("style");
            var left = style["left"];
            var top_1 = style["top"];
            var width = style["width"];
            var height = style["height"];
            this.canvasContext.clearRect(parseInt(left, 10), parseInt(top_1, 10), parseInt(width, 10), parseInt(height, 10));
            this.canvasContext.strokeRect(parseInt(left, 10), parseInt(top_1, 10), parseInt(width, 10), parseInt(height, 10));
            this.canvasContext.fillRect(parseInt(left, 10), parseInt(top_1, 10), parseInt(width, 10), parseInt(height, 10));
        }
    };
    /**
     * Remove image at given index.
     * @param index the index in screenshot array.
     */
    FeedbackScreenshotComponent.prototype.remove = function (index) {
        this.screenshots.splice(index, 1);
        this.changeDetection.detectChanges();
    };
    /**
     * Remove the mouse drawing events.
     */
    FeedbackScreenshotComponent.removeMouseDrawingEvents = function () {
        document.removeEventListener("mousedown");
        document.removeEventListener("mouseup");
        document.removeEventListener("mousemove");
    };
    /**
     * Setup mouse drawing events.
     */
    FeedbackScreenshotComponent.prototype.setupMouseDrawingEvents = function () {
        var _this = this;
        document.addEventListener("mousedown", function (event) {
            /* Make sure we don't execute when clicking on feedback buttons. */
            if (!event.target || !event.target.classList.contains(FEEDBACK_BTN_CLASS)) {
                _this.startDrawing(event.pageX, event.pageY);
            }
        });
        document.addEventListener("mouseup", function (event) {
            /* Make sure we don't execute when clicking on feedback buttons. */
            if (!event.target || !event.target.classList.contains(FEEDBACK_BTN_CLASS)) {
                _this.finishDrawing(event.pageX, event.pageY);
            }
        });
        document.addEventListener("mousemove", function (event) {
            _this.adjustDrawing(event.pageX, event.pageY);
        });
    };
    /**
     * Start drawing from given position.
     * @param x the x-axis page position to start at.
     * @param y the y-axis page position to start at.
     */
    FeedbackScreenshotComponent.prototype.startDrawing = function (x, y) {
        this.centerX = x;
        this.centerY = y;
        this.canvasContext.beginPath();
        this.isDrawing = true;
        this.redraw();
    };
    /**
     * Capture a screenshot.
     */
    FeedbackScreenshotComponent.prototype.takeScreenshot = function () {
        var _this = this;
        this.preRenderScrollPosition = window.pageYOffset;
        document.body.scrollTop = 0; // Scroll to the top.
        document.body.style["overflow"] = "hidden"; // Disable Scroll.
        document.body.classList.add("wait"); // Set cursor for body to wait.
        this.canvas.classList.add("wait"); // Set cursor for canvas to wait.
        this.feedback.lockTakeScreenshotButtons();
        FeedbackScreenshotComponent.removeMouseDrawingEvents();
        var options = {
            onrendered: function (canvas) { return _this.processRenderedScreenshot(canvas); }
        };
        /* Create a canvas from body, and create an image out of it. */
        html2canvas(document.body, options);
    };
    return FeedbackScreenshotComponent;
}());
export { FeedbackScreenshotComponent };
FeedbackScreenshotComponent.decorators = [
    { type: Component, args: [{
                selector: "ngx-bootstrap-feedback-screenshot",
                templateUrl: "./feedback-screenshot.html",
            },] },
];
/** @nocollapse */
FeedbackScreenshotComponent.ctorParameters = function () { return [
    { type: ChangeDetectorRef, },
    { type: FeedbackService, },
]; };
FeedbackScreenshotComponent.propDecorators = {
    'configuration': [{ type: Input },],
};
//# sourceMappingURL=feedback-screenshot.component.js.map